(function(){

require("Ice");
require("Glacier2");
require("./Printer");

var communicator;

process.once("SIGINT", function() {
    if(communicator)
    {
        communicator.destroy().finally(
            function()
            {
                process.exit(0);
            });
    }
});

Ice.Promise.try(
    function()
    {
        var id = new Ice.InitializationData();
        id.properties = Ice.createProperties();
        id.properties.setProperty("Ice.Default.EncodingVersion", "1.0");
        id.properties.setProperty("Ice.Default.Router", "MyGlacier2/router:tcp -h localhost -p 4063");
        communicator = Ice.initialize(id);

        function createSession()
        {
            return Ice.Promise.try(
                function()
                {
                    var router = communicator.getDefaultRouter();
                    var id;

                    return Glacier2.RouterPrx.checkedCast(router).then(
                        function(proxy)
                        {
                            router = proxy
                            console.log("This Client accepts any user-id / password combination.");
                            process.stdout.write("user id : ");
                            return getline();
                        }
                    ).then(
                        function(str)
                        {
                            id = str;
                            process.stdout.write("password : ");
                            return getline();
                        }
                    ).then(
                        function(password)
                        {
                            return router.createSession(id, password);
                        }
                    ).then(
                        function(session)
                        {
                            return runWithSession(router, Demo.PrinterPrx.uncheckedCast(session));         
                        },
                        function(ex)
                        {
                            if(ex instanceof Glacier2.PermissionDeniedException)
                            {
                                console.log("permission denied :\n" + ex.reason);
                                return createSession();
                            }
                            else if(ex instanceof Glacier2.CannotCreateSessionException)
                            {
                                console.log("cannot create session :\n" + ex.reason);
                                return createSession();
                            }
                            else
                            {
                                throw ex;
                            }
                        });
                });
        };

        function runWithSession(router, session)
        { 
            var proxy = communicator.stringToProxy("printer -t:tcp -h localhost -p 10000");
            var printer = Demo.PrinterPrx.uncheckedCast(proxy)

            Ice.Promise.all(
                printer.printString('my string').then(
                    function(){
                        console.log("printString well-executed");
                    }
                ),
                printer.getTest().then(
                    function(result){
                        console.log(result);
                    }
                ),
                printer.getInfo().then(
                    function(result){
                        console.log(result.str);
                    }
                ),
                printer.getMaster().then(
                    function(result){
                        masterPrx = Demo.MasterPrx.uncheckedCast(result);   
                        //console.log(masterPrx.getMasterString().toString());

                        masterPrx.getMasterString().then(
                            function(str){
                                console.log(str);
                            },
                            function(ex){
                                console.log("fail : "+ex);
                            }
                        )
                        //return result;
                    }
                )/*.then(
                    function(master){
                        return master.getMasterString().then(
                            function(str){
                                console.log("str : "+str);
                            },
                            function(ex){
                                console.log(ex);
                            }
                        )       
                    }
                )*/
            );
        }
        return createSession();
    }
).finally(
    function()
    {
        if(communicator)
        {
            return communicator.destroy();
        }
    }
).then(
    function()
    {
        process.exit(0);
    },
    function(ex)
    {
        console.log(ex.toString());
        process.exit(1);
    });

var getline = function()
{
    var p = new Ice.Promise();
    process.stdin.resume();
    process.stdin.once("data", 
        function(buffer)
        {
            process.stdin.pause();
            p.succeed(buffer.toString("utf-8").trim());
        });
    return p;
};

}());
